﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using C1.WPF.C1Chart;

namespace WPF_Chart_Tooltips_BitmapMode
{
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window, INotifyPropertyChanged
  {
    public ObservableCollection<DataItem> DataItems { get; private set; }
    public Dictionary<int, DataItem> ItemsDictionary = new Dictionary<int, DataItem>();

    private static readonly Random RandomGenerator = new Random();

    private const int NumPoints = 3000;

    private readonly Timer _timer;

    public MainWindow()
    {
      InitializeComponent();

      DataContext = this;

      DataItems = new ObservableCollection<DataItem>();

      InitializeData();

      AddTooltipLayer(_scatterPlot);

      _timer = new Timer(OnTimerUpdate, null, 10000, 10000);

    }

    void AddTooltipLayer(C1Chart chart)
    {
        var pnl = new ChartPanel();

        var obj = new ChartPanelObject()
        {
        HorizontalAlignment = HorizontalAlignment.Right,
        VerticalAlignment = VerticalAlignment.Bottom
        };

        var bdr = new Border()
        {
        Background = new SolidColorBrush(Colors.Blue) { Opacity = 0.6 },
        BorderBrush = new SolidColorBrush(Colors.Blue),
        BorderThickness = new Thickness(1, 1, 3, 3),
        CornerRadius = new CornerRadius(6, 6, 0, 6),
        Padding = new Thickness(3)
        };

        var sp = new StackPanel();

        var tb1 = new TextBlock();
        var bind1 = new Binding();
        bind1.Source = obj;
        bind1.StringFormat = "x={0:#.##}";
        bind1.Path = new PropertyPath("DataPoint.X");
        tb1.SetBinding(TextBlock.TextProperty, bind1);

        var tb2 = new TextBlock();
        var bind2 = new Binding();
        bind2.Source = obj;
        bind2.StringFormat = "y={0:#.##}";
        bind2.Path = new PropertyPath("DataPoint.Y");
        tb2.SetBinding(TextBlock.TextProperty, bind2);

        sp.Children.Add(tb1);
        sp.Children.Add(tb2);

        bdr.Child = sp;

        obj.Content = bdr;
        obj.DataPoint = new Point();
        obj.Action = ChartPanelAction.MouseMove;
        obj.Attach = ChartPanelAttach.DataXY;

        pnl.Children.Add(obj);

        chart.View.Layers.Add(pnl);
    }

    private void InitializeData()
    {
      for (var i = 0; i < NumPoints; ++i)
      {
        var item = new DataItem();
        item["XValue"] = RandomGenerator.NextDouble();

        item["YValue"] = RandomGenerator.NextDouble();

        ItemsDictionary[i] = item;
        DataItems.Add(item);
      }
    }

    private void OnTimerUpdate(object state)
    {
      for (var i = 0; i < NumPoints; ++i)
      {
        DataItem item;
        if (ItemsDictionary.TryGetValue(i, out item))
        {
          item["XValue"] = RandomGenerator.NextDouble();
          item["YValue"] = RandomGenerator.NextDouble();
        }
      }

      Toolbox.DispatchToUI(() =>
                               {
                                 DataItems = new ObservableCollection<DataItem>(ItemsDictionary.Values);
                                 OnPropertyChanged("DataItems");
                               });
    }

    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged(string propertyName)
    {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }
  }
}
